/*
 * Chapter 21
 * Task 4: Handling pushbuttons with interrupts, button debouncing
 *
 * The 3 LEDs connected to PA0,1,2 pins show a 3 digit counter
 * With pushbuttons connected to PD2 and PD3 the value can be incremented or decremented
 * When a button is released, the rising edge on the input generates an interrupt
 * Timer1 is used for button debouncing 
 */

#include "../Headers/main.h"

volatile uint8_t cntr = 0;

//Interrupt for increment button
ISR(INT0_vect)
{
	//Disable INTR0 interrupt
	cbi(GICR, INT0);

	//Reset Timer1 counter
	TCNT1 = 0;
	//Enable Timer1 overflow interrupt
	sbi(TIMSK, TOIE1);
	
	//If counter smaller than 7 (maximum)
	if (cntr < 7)
	{
		//Increment
		cntr++;
	}
}

//Interrupt for decrement button
ISR(INT1_vect)
{
	//Disable INTR1 interrupt
	cbi(GICR, INT1);
	
	//Reset Timer1 counter
	TCNT1 = 0;
	//Enable Timer1 overflow interrupt
	sbi(TIMSK, TOIE1);
	
	//If counter larger than 0 (minimum)
	if (cntr > 0)
	{
		//Decrement
		cntr--;
	}
}

//Interrupt for Timer1 Overflow
ISR(TIMER1_OVF_vect)
{
	//Clear External interrupt flags, see page 68
	sbi(GIFR, INTF0);
	sbi(GIFR, INTF1);

	//Disable Timer1 overflow interrupt
	cbi(TIMSK, TOIE1);
	
	//Enable External interrupts, see page 67
	sbi(GICR, INT0);
	sbi(GICR, INT1);
}


int main(void)
{
	//Initializing IO ports
	IOInit();
	//Initializing Timer1 peripheral
	TimerInit();

	//Infinite loop
	while (1)
	{
		//Set LED states connected to PA0,1,2 to be equal with counter
		PORTA = cntr;
	}

	return (0);
}
